#include "stdafx.h"
#include "resource.h"
#include "Settings.h"
#include ".\Launcher.h"

CLauncher::CLauncher(IProcessEventCallback *callback) :
		m_dwProcessId(0), m_hProcess(NULL),
		m_callback(callback),
		m_hStdInput(INVALID_HANDLE_VALUE), m_hStdOutput(INVALID_HANDLE_VALUE),
		m_hStdError(INVALID_HANDLE_VALUE),
		m_Stopped(false)
{
	::InitializeCriticalSection(&m_cs);
}

CLauncher::~CLauncher(void)
{
	CloseStdHandle();
	::DeleteCriticalSection(&m_cs);
}

void CLauncher::CloseStdHandle()
{
	if (m_hStdInput != INVALID_HANDLE_VALUE)	::CloseHandle(m_hStdInput);
	if (m_hStdOutput != INVALID_HANDLE_VALUE)	::CloseHandle(m_hStdOutput);
	if (m_hStdError != INVALID_HANDLE_VALUE)	::CloseHandle(m_hStdError);
	m_hStdInput  = INVALID_HANDLE_VALUE;
	m_hStdOutput = INVALID_HANDLE_VALUE;
	m_hStdError  = INVALID_HANDLE_VALUE;
}

/*
BOOL WINAPI CLauncher::HandlerRoutine(DWORD dwCtrlType)
{
	switch (dwCtrlType) {
	case CTRL_C_EVENT:
	case CTRL_BREAK_EVENT:
OutputDebugString(_T("CTRL_X_EVENT\n"));
		break;
	case CTRL_CLOSE_EVENT:
OutputDebugString(_T("CTRL_CLOSE_EVENT\n"));
		FreeConsole();
	//	AllocConsole();
		return FALSE;
	default:
		return FALSE;
	}
	return TRUE;
}
*/

BOOL CLauncher::Launch(const CSettings& settings, int SettingNum,
		const CString& strInput, const CString& strOutput)
{
	ATLTRACE(_T("Launch()\n"));
	BOOL ret;
	PROCESS_INFORMATION pi;
	STARTUPINFO si = {sizeof(si)};
	
	::AllocConsole();
	::SetConsoleCtrlHandler(NULL, TRUE);	// Ignore Ctrl+C
//	::SetConsoleCtrlHandler(HandlerRoutine, TRUE);	// Ignore Ctrl+C
//	::ShowWindow(::GetConsoleWindow(), SW_HIDE);
	::SetConsoleTitle(_T("TsConvGUI Log Window"));
	
	CloseStdHandle();
	
	HANDLE hStdInputWrite, hStdInputRead;
	HANDLE hStdOutputRead, hStdOutputWrite;
	HANDLE hStdErrorRead, hStdErrorWrite;
	
	ret = CreateStdIoPipes(hStdInputWrite, hStdInputRead,
			hStdOutputRead, hStdOutputWrite,
			hStdErrorRead, hStdErrorWrite);
	if (!ret) {
		return FALSE;
	}
	
	si.dwFlags = STARTF_USESTDHANDLES;
	si.hStdInput  = hStdInputRead;
	si.hStdOutput = hStdOutputWrite;
	si.hStdError  = hStdErrorWrite;
//	si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
//	si.hStdError  = GetStdHandle(STD_ERROR_HANDLE);
	m_hStdInput  = hStdInputWrite;
	m_hStdOutput = hStdOutputRead;
	m_hStdError  = hStdErrorRead;
	
	
	{
		LPCTSTR fawenv;
		CString fawcl = settings.m_FAW;
		fawcl.MakeLower();
		if (fawcl.Find(_T("fawcl.exe")) >= 0) {
			fawenv = _T("FAWCL");
		} else {
			fawenv = _T("FAW");
		}

		// Set Environments
		::SetEnvironmentVariable(_T("TsSplitter"), settings.m_TsSplitter);
		::SetEnvironmentVariable(_T("make_gl"), settings.m_make_gl);
		::SetEnvironmentVariable(_T("ts2aac"), settings.m_ts2aac);
		::SetEnvironmentVariable(_T("aactool"), settings.m_aactool);
		::SetEnvironmentVariable(fawenv, settings.m_FAW);
		::SetEnvironmentVariable(_T("CAP2ASS"), settings.m_CAP2ASS);

		CString exec_ts2aac;
		exec_ts2aac.Format(_T("\"%s%s\""), settings.m_ExeDirectory, _T("exec_ts2aac.exe"));
		::SetEnvironmentVariable(_T("EXEC_TS2AAC"), exec_ts2aac);

		::SetEnvironmentVariable(_T("EXIT_IF_ERROR"), _T("1"));
	}
	
	CString strCmdLine;
	{
		// Make command line
		const CSettingItem& item = settings.m_SettingList[SettingNum];
		
		TCHAR cmd[MAX_PATH];
		if (::GetEnvironmentVariable(_T("ComSpec"), cmd, _countof(cmd)) == 0) {
			lstrcpy(cmd, _T("cmd.exe"));
		}
		
		CString bat;
		if (::PathIsRelative(item.m_BatchFile)) {
			bat = settings.m_ExeDirectory + item.m_BatchFile;
		} else {
			bat = item.m_BatchFile;
		}
		CString opt = _T("");
		if (!item.m_Use_TsSplitter)
			opt += _T(" -nosplit");
		if (!item.m_Use_make_gl)
			opt += _T(" -nogl");
		if (!item.m_Use_ts2aac)
			opt += _T(" -noaac");
		if (!item.m_Use_aactool)
			opt += _T(" -noaactool");
		if (!item.m_Use_FAW)
			opt += _T(" -nofaw");
		if (!item.m_Use_CAP2ASS)
			opt += _T(" -nocap");
		if (!item.m_TsSplitter_opt.IsEmpty())
			opt += _T(" -splitopt \"") + item.m_TsSplitter_opt + _T('"');
		if (!item.m_ts2aac_opt.IsEmpty())
			opt += _T(" -aacopt \"") + item.m_ts2aac_opt + _T('"');
		if (!item.m_CAP2ASS_opt.IsEmpty())
			opt += _T(" -capopt \"") + item.m_CAP2ASS_opt + _T('"');
		if (!item.m_options.IsEmpty())
			opt += _T(" ") + item.m_options;
		
		strCmdLine.Format(_T("\"%s\" /c \"\"%s\" \"%s\" \"%s\"%s\""), cmd, bat, strInput, strOutput, opt);
		ATLTRACE(_T("CmdLine: %s\n"), strCmdLine);
	}
	LPTSTR szCmdLine = strCmdLine.GetBuffer(0);
	if (!::CreateProcess(NULL, szCmdLine, NULL, NULL, TRUE,
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi)) {
		goto error;
	}
	m_hProcess = pi.hProcess;
	m_dwProcessId = pi.dwProcessId;
//	::ResumeThread(pi.hThread);
	::CloseHandle(pi.hThread);
	
	::CloseHandle(hStdInputRead);
	::CloseHandle(hStdOutputWrite);
	::CloseHandle(hStdErrorWrite);
	
	m_Stopped = false;
	
	HANDLE hThread = ::CreateThread(NULL, 0, WaitProcess, this, 0, NULL);
	::CloseHandle(hThread);
	HANDLE hThread2 = ::CreateThread(NULL, 0, WaitStdError, this, 0, NULL);
	::CloseHandle(hThread2);
	
	return TRUE;
	
error:
	::CloseHandle(hStdInputWrite);
	::CloseHandle(hStdInputRead);
	::CloseHandle(hStdOutputRead);
	::CloseHandle(hStdOutputWrite);
	::CloseHandle(hStdErrorRead);
	::CloseHandle(hStdErrorWrite);
	m_hStdInput  = INVALID_HANDLE_VALUE;
	m_hStdOutput = INVALID_HANDLE_VALUE;
	m_hStdError  = INVALID_HANDLE_VALUE;
	return FALSE;
}


BOOL CLauncher::CreateStdIoPipes(HANDLE& hStdInputWrite, HANDLE& hStdInputRead,
		HANDLE& hStdOutputRead, HANDLE& hStdOutputWrite,
		HANDLE& hStdErrorRead, HANDLE& hStdErrorWrite)
{
	BOOL ret;
	SECURITY_ATTRIBUTES sa = {sizeof(sa)};
	sa.bInheritHandle = TRUE;
	sa.lpSecurityDescriptor = NULL;
	
	HANDLE hCurProcess = ::GetCurrentProcess();
	
	hStdInputWrite  = INVALID_HANDLE_VALUE;
	hStdInputRead   = INVALID_HANDLE_VALUE;
	hStdOutputRead  = INVALID_HANDLE_VALUE;
	hStdOutputWrite = INVALID_HANDLE_VALUE;
	hStdErrorRead   = INVALID_HANDLE_VALUE;
	hStdErrorWrite  = INVALID_HANDLE_VALUE;
	
	// StdInput
	HANDLE hStdInputWriteTemp;
	ret = ::CreatePipe(&hStdInputRead, &hStdInputWriteTemp, &sa, 0);
	if (!ret) {
		goto error;
	}
	ret = ::DuplicateHandle(hCurProcess, hStdInputWriteTemp,
			hCurProcess, &hStdInputWrite, 0, FALSE, DUPLICATE_SAME_ACCESS);
	::CloseHandle(hStdInputWriteTemp);
	if (!ret) {
		goto error;
	}
	
	// StdOutput
	HANDLE hStdOutputReadTemp;
	ret = ::CreatePipe(&hStdOutputReadTemp, &hStdOutputWrite, &sa, 0);
	if (!ret) {
		goto error;
	}
	ret = ::DuplicateHandle(hCurProcess, hStdOutputReadTemp,
			hCurProcess, &hStdOutputRead, 0, FALSE, DUPLICATE_SAME_ACCESS);
	::CloseHandle(hStdOutputReadTemp);
	if (!ret) {
		goto error;
	}
	
	// StdError
	HANDLE hStdErrorReadTemp;
	ret = ::CreatePipe(&hStdErrorReadTemp, &hStdErrorWrite, &sa, 0);
	if (!ret) {
		goto error;
	}
	ret = ::DuplicateHandle(hCurProcess, hStdErrorReadTemp,
			hCurProcess, &hStdErrorRead, 0, FALSE, DUPLICATE_SAME_ACCESS);
	::CloseHandle(hStdErrorReadTemp);
	if (!ret) {
		goto error;
	}
/*
	ret = ::DuplicateHandle(hCurProcess, hStdOutputWrite,
			hCurProcess, &hStdErrorWrite, 0, TRUE, DUPLICATE_SAME_ACCESS);
	if (!ret) {
		goto error;
	}
	ret = ::DuplicateHandle(hCurProcess, hStdOutputRead,
			hCurProcess, &hStdErrorRead, 0, FALSE, DUPLICATE_SAME_ACCESS);
	if (!ret) {
		goto error;
	}
*/
	
	return TRUE;
	
error:
	if (hStdInputWrite  != INVALID_HANDLE_VALUE)	::CloseHandle(hStdInputWrite);
	if (hStdInputRead   != INVALID_HANDLE_VALUE)	::CloseHandle(hStdInputRead);
	if (hStdOutputRead  != INVALID_HANDLE_VALUE)	::CloseHandle(hStdOutputRead);
	if (hStdOutputWrite != INVALID_HANDLE_VALUE)	::CloseHandle(hStdOutputWrite);
	if (hStdErrorRead   != INVALID_HANDLE_VALUE)	::CloseHandle(hStdErrorRead);
	if (hStdErrorWrite  != INVALID_HANDLE_VALUE)	::CloseHandle(hStdErrorWrite);
	hStdInputWrite  = INVALID_HANDLE_VALUE;
	hStdInputRead   = INVALID_HANDLE_VALUE;
	hStdOutputRead  = INVALID_HANDLE_VALUE;
	hStdOutputWrite = INVALID_HANDLE_VALUE;
	hStdErrorRead   = INVALID_HANDLE_VALUE;
	hStdErrorWrite  = INVALID_HANDLE_VALUE;
	return FALSE;
}

BOOL ExitRemoteProcess(HANDLE hProcess, UINT uExitCode)
{
	LPTHREAD_START_ROUTINE pfnExitProcess = 
			(LPTHREAD_START_ROUTINE) GetProcAddress(
					GetModuleHandle(_T("kernel32.dll")), "ExitProcess");
	if ((hProcess != NULL) && (pfnExitProcess != NULL)) {
		HANDLE hThread = CreateRemoteThread(hProcess, NULL, 0,
				pfnExitProcess, (LPVOID) uExitCode, 0, NULL);
		if (hThread != NULL) {
			CloseHandle(hThread);
			return TRUE;
		}
	}
	return FALSE;
}


BOOL CLauncher::StopProcess()
{
	ATLTRACE(_T("StopProcess()\n"));
	Lock();
	if (m_hProcess != NULL) {
		m_Stopped = true;
		HWND hwnd = ::GetConsoleWindow();
		if (hwnd != NULL) {
			::FreeConsole();
			::PostMessage(hwnd, WM_CLOSE, 0, 0);
		}
		
	//	ExitRemoteProcess(m_hProcess, 1);
		
	}
/*
	if (m_hProcess != NULL) {
		DWORD dwWritten;
		::GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, m_dwProcessId);
		ATLTRACE(_T("GenerateConsoleCtrlEvent() OK\n"));
		::WriteFile(m_hStdInput, " Y\n", 3, &dwWritten, NULL);
		ATLTRACE(_T("WriteFile() OK\n"));
		Unlock();
		
		::Sleep(300);
		ATLTRACE(_T("Sleep() OK\n"));
		
		Lock();
		if (m_hProcess != NULL) {
			DWORD exitcode = 0;
			::GetExitCodeProcess(m_hProcess, &exitcode);
			ATLTRACE(_T("GetExitCodeProcess() OK\n"));
			if (exitcode == STILL_ACTIVE) {
				::GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, m_dwProcessId);
				::WriteFile(m_hStdInput, "Y\n", 2, &dwWritten, NULL);
			//	::TerminateProcess(m_hProcess, 1);
			}
		}
		m_Stopped = true;
	}
*/
	CString strText;
	strText.LoadString(IDS_STOP_PROCESS);
	m_callback->Progress(strText, 0, 100);
	Unlock();
	return TRUE;
}

DWORD WINAPI CLauncher::WaitProcess(LPVOID param)
{
	return reinterpret_cast<CLauncher *>(param)->WaitProcess();
}
DWORD CLauncher::WaitProcess()
{
	BOOL ret1 = TRUE, ret2 = TRUE;
	CHAR buf[512];
	int buflen = 0;
	while (ret1 || ret2) {
		DWORD avail, cb, cb2;
		ret1 = ::PeekNamedPipe(m_hStdOutput, NULL, 0, NULL, &avail, NULL);
		if (ret1 && (avail > 0)) {
			::ReadFile(m_hStdOutput, buf + buflen, min(avail, sizeof(buf)/2 - 1), &cb, NULL);
			buf[buflen + cb] = '\0';
			::WriteFile(::GetStdHandle(STD_OUTPUT_HANDLE), buf + buflen, cb, &cb2, NULL);
			
			LPSTR p = buf;
			LPSTR pEOL = p + ::StrCSpnA(p, "\r\n");
			while (*pEOL != '\0') {
				if ((*pEOL == '\r') && (*(pEOL + 1) == '\n')) {
					++pEOL;
				}
				CString str(p, pEOL - p + 1);
				ParseLine(str);
				p = ++pEOL;
				pEOL = p + ::StrCSpnA(p, "\r\n");
			}
			buflen = ::lstrlenA(p);
			if (buflen + 1 >= sizeof(buf) / 2) {
				ParseLine(p);
				buflen = 0;
			} else {
				memmove(buf, p, buflen + 1);
			}
			
			
		/*
			LPSTR p = ::StrStrA(buf, "\n===>> ");
			if (p != NULL) {
				p += 7;
				LPSTR q = p;
				while (p < buf + cb) {
					if (*p == '\r') {
						break;
					}
					++p;
				}
				CString str(q, p - q);
				m_callback->Progress(str, 0, 100);
			} else if ((p = ::StrStrA(buf, " MB) ")) != NULL) {
				int pos = ::StrToIntA(p + 5);
				m_callback->Progress(NULL, pos, 100);
			}
		*/
		}
		ret2 = ::PeekNamedPipe(m_hStdError, NULL, 0, NULL, &avail, NULL);
	/*
		if (ret2 && (avail > 0)) {
			::ReadFile(m_hStdError, buf, min(avail, sizeof(buf)), &cb, NULL);
			::WriteFile(::GetStdHandle(STD_ERROR_HANDLE), buf, cb, &cb, NULL);
		}
	*/
		::Sleep(10);
	}
	if (buf[0] != '\0') {
		ParseLine(buf);
	}
	::WaitForSingleObject(m_hProcess, INFINITE);
	ATLTRACE(_T("WaitForSingleObject() End\n"));
	
	DWORD exitcode = 0;
	::GetExitCodeProcess(m_hProcess, &exitcode);
	
//	::FreeConsole();
	
	Lock();
	::CloseHandle(m_hProcess);
	m_hProcess = NULL;
	m_dwProcessId = 0;
	Unlock();
	
	m_callback->ProcessExit(exitcode, m_Stopped);
	CloseStdHandle();
	return 0;
}

void CLauncher::ParseLine(CString strLine)
{
	int n;
	if (strLine.Find(_T("===>> ")) == 0) {
		// reset the progress bar
		CString str = strLine.Mid(6);
		str.TrimRight();
		m_callback->Progress(str, 0, 100);
	} else if ((n = strLine.Find(_T(" MB) "))) > 0) {
//OutputDebugString(LPCTSTR(strLine) + n + 5);
		// Parse output of TsSpiltter
		// "  ( x,xxx MB/ x,xxx MB) xx%"
	//	int pos = ::StrToInt(LPCTSTR(strLine) + n + 5);
	//	m_callback->Progress(NULL, pos, 100);
		strLine.Remove(_T(','));
		n = strLine.Find(_T(" ( "));
		int pos = ::StrToInt(LPCTSTR(strLine) + n + 3);
		n = strLine.Find(_T(" MB/ "));
		int total = ::StrToInt(LPCTSTR(strLine) + n + 5);
		m_callback->Progress(NULL, pos, total);
	} else if (strLine.Find(_T("")) == 0) {
		// fawcl: "󋵁@F xx"
		// Caption2Ass_mod1: "󋵁Fxx% (nn)"
		CString str(_T("F"));
		n = strLine.Find(str);
		if (n >= 0) {
			n += str.GetLength();
			LPCTSTR p = LPCTSTR(strLine) + n;
			while (*p == _T(' '))
				++p;
			int pos = ::StrToInt(p);
			m_callback->Progress(NULL, pos, 100);
		}
	} else if ((strLine[0] == _T(' ')) && (::StrStr(strLine, _T("%\r")) != NULL)) {
//OutputDebugString(LPCTSTR(strLine));
		// Parse output of ROBOCOPY
		LPCTSTR p = strLine;
		while (*p == _T(' '))
			++p;
		int pos = ::StrToInt(p) * 10;
		while (*p != _T('.'))
			++p;
		pos += ::StrToInt(++p);
		m_callback->Progress(NULL, pos, 1000);
	}
}


DWORD WINAPI CLauncher::WaitStdError(LPVOID param)
{
	return reinterpret_cast<CLauncher *>(param)->WaitStdError();
}
DWORD CLauncher::WaitStdError()
{
	BOOL ret = TRUE;
	while (ret) {
		DWORD avail, cb, cb2;
		CHAR buf[256];
		ret = ::PeekNamedPipe(m_hStdError, NULL, 0, NULL, &avail, NULL);
		if (ret && (avail > 0)) {
			::ReadFile(m_hStdError, buf, min(avail, sizeof(buf)-1), &cb, NULL);
			buf[cb] = '\0';
			::WriteFile(::GetStdHandle(STD_ERROR_HANDLE), buf, cb, &cb2, NULL);
//OutputDebugStringA(buf);
			// Parse output of ts2aac
			LPSTR p = (buf[0] == '\r') ? buf + 1 : buf;
			int pos = ::StrToIntA(p);
			if (pos > 0) {
				m_callback->Progress(NULL, pos, 1000);
			}
		}
		::Sleep(10);
	}
	return 0;
}
