// MainDlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#define WM_PROCESS_EXIT	(WM_USER+1)


class CMainDlg : public CDialogImpl<CMainDlg>, public CUpdateUI<CMainDlg>,
		public CWinDataExchange<CMainDlg>, public CDragListNotifyImpl<CMainDlg>,
		public CMessageFilter, public CIdleHandler,
		public IProcessEventCallback
{
public:
	enum { IDD = IDD_MAINDLG };

	CString m_szInputPath;
	CString m_szOutputPath;
	CString m_szDefaultPath;
	CProgressBarCtrl m_progress;
	CComPtr<ITaskbarList3> m_ptrTaskbarList3;
	CComboBox m_combo_enc;
	CDragListBox m_batch_list;
	CSettings m_Settings;

	CLauncher m_Launcher;
	BOOL m_Converting;
	BOOL m_Dragging;

	// RXgN^
	CMainDlg() : m_Launcher(this), m_Converting(FALSE), m_Dragging(FALSE)
	{
	}

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		return CWindow::IsDialogMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_DDX_MAP(CMainDlg)
		DDX_CONTROL_HANDLE(IDC_PROGRESS, m_progress);
		DDX_CONTROL_HANDLE(IDC_COMBO_ENC, m_combo_enc);
		DDX_CONTROL_HANDLE(IDC_BATCH_LIST, m_batch_list);
		DDX_TEXT(IDC_EDIT_INPUT, m_szInputPath)
		DDX_TEXT(IDC_EDIT_OUTPUT, m_szOutputPath)
		DDX_TEXT(IDC_EDIT_FOLDER, m_szDefaultPath)
	END_DDX_MAP()

	BEGIN_UPDATE_UI_MAP(CMainDlg)
	END_UPDATE_UI_MAP()

	BEGIN_MSG_MAP(CMainDlg)
		MSG_WM_INITDIALOG(OnInitDialog)
		MSG_WM_DESTROY(OnDestroy)
		MSG_WM_DROPFILES(OnDropFiles)
		MSG_WM_SYSCOMMAND(OnSysCommand)
	//	COMMAND_ID_HANDLER_EX(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER_EX(IDOK, OnOK)
		COMMAND_ID_HANDLER_EX(IDCANCEL, OnCancel)
		COMMAND_ID_HANDLER_EX(IDC_BROWSE_INPUT, OnBrowseInput)
		COMMAND_ID_HANDLER_EX(IDC_BROWSE_OUTPUT, OnBrowseOutput)
		COMMAND_ID_HANDLER_EX(IDC_BROWSE_FOLDER, OnBrowseFolder)
		COMMAND_ID_HANDLER_EX(IDC_BUTTON_ENC_SET, OnButtonEncSet)
		COMMAND_ID_HANDLER_EX(IDC_BATCH_ADD, OnBatchAdd)
		COMMAND_ID_HANDLER_EX(IDC_BATCH_DEL, OnBatchDel)
		COMMAND_ID_HANDLER_EX(IDC_BATCH_REPLACE, OnBatchReplace)
		COMMAND_ID_HANDLER_EX(IDC_BATCH_DEL_ALL, OnBatchDelAll)
		COMMAND_HANDLER_EX(IDC_BATCH_LIST, LBN_SELCHANGE, OnBatchListSelChange)
		COMMAND_HANDLER_EX(IDC_COMBO_ENC, CBN_SELCHANGE, OnComboSelChange)
		MESSAGE_HANDLER(WM_PROCESS_EXIT, OnProcessExit)
		CHAIN_MSG_MAP(CDragListNotifyImpl<CMainDlg>)
	END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	BOOL OnInitDialog(CWindow wndFocus, LPARAM lInitParam)
	{
		// center the dialog on the screen
		CenterWindow();

	//	::AllocConsole();
	//	CWindow wndConsole = ::GetConsoleWindow();
	//	wndConsole.ShowWindow(SW_HIDE);
	//	::SetConsoleCtrlHandler(NULL, TRUE);	// Ignore Ctrl+C

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME),
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		SetIcon(hIcon, TRUE);
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME),
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		// set the system menu
		CMenuHandle sysMenu = ::GetSystemMenu(m_hWnd, FALSE);
		sysMenu.AppendMenu(MF_SEPARATOR);
		CString strMenuAbout;
		strMenuAbout.LoadString(IDS_APP_ABOUT);
		sysMenu.AppendMenu(MF_STRING, ID_APP_ABOUT, strMenuAbout);

		// Load Settings
		m_Settings.Load();
		m_szDefaultPath = m_Settings.m_DefaultOutputPath;

		// set controls
		DoDataExchange(DDX_LOAD);

		m_progress.SetRange(0, 1000);
		m_ptrTaskbarList3.CoCreateInstance(CLSID_TaskbarList);

		m_batch_list.SetHorizontalExtent(1200);		// KɍL
		m_batch_list.MakeDragList();

		int count = m_Settings.m_SettingList.GetSize();
		for (int i = 0; i < count; i++) {
			m_combo_enc.AddString(m_Settings.m_SettingList[i].m_SettingName);
		}
		m_combo_enc.SetCurSel(m_Settings.m_SelectNum);

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		UIAddChildWindowContainer(m_hWnd);

		return TRUE;
	}

	void OnDestroy()
	{
		// unregister message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->RemoveMessageFilter(this);
		pLoop->RemoveIdleHandler(this);
	}

	void OnSysCommand(UINT nID, CPoint /*point*/)
	{
		if (nID == ID_APP_ABOUT) {
			CAboutDlg dlg;
			dlg.DoModal();
		} else {
			SetMsgHandled(FALSE);
		}
	}

	void OnDropFiles(HDROP hDropInfo)
	{
		DoDataExchange(DDX_SAVE);
		m_szInputPath.Empty();
		m_szOutputPath.Empty();
		UINT nMax = ::DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
		for (UINT i = 0; i < nMax; i++) {
			TCHAR file[MAX_PATH];
			::DragQueryFile(hDropInfo, i, file, _countof(file));
			ATLTRACE(_T("%s\n"), file);
			AddFile(file);
		}
		::DragFinish(hDropInfo);
		DoDataExchange(DDX_LOAD);
	}

	LRESULT OnOK(UINT uNotifyCode, int nID, CWindow wndCtl)
	{
		if (m_Converting) {
			m_Launcher.StopProcess();
			return 0;
		}
		if (m_batch_list.GetCount() == 0) {
			OnBatchAdd(uNotifyCode, nID, wndCtl);
		}
		m_batch_list.SetCurSel(0);
		StartConvert();
		return 0;
	}
	
	void StartConvert()
	{
		int sel = m_batch_list.GetCurSel();
		if (sel == LB_ERR) {
			return;
		}
		CString str;
		m_batch_list.GetText(sel, str);
		LoadListItem(str);
		
		int cbsel = m_combo_enc.GetCurSel();
		if (cbsel == CB_ERR) {
			return;
		}
		BOOL ret = m_Launcher.Launch(m_Settings, cbsel, m_szInputPath, m_szOutputPath);
		LockUI(ret);
	}

	LRESULT OnCancel(UINT /*uNotifyCode*/, int nID, CWindow /*wndCtl*/)
	{
		DoDataExchange(DDX_SAVE);
		m_Settings.m_DefaultOutputPath = m_szDefaultPath;
		m_Settings.Save();

		CloseDialog(nID);
	//	CWindow wndConsole = ::GetConsoleWindow();
	//	wndConsole.SendMessage(WM_CLOSE, 0, 0);
//		::FreeConsole();
		return 0;
	}

	LRESULT OnBrowseInput(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		DoDataExchange(DDX_SAVE);
		CString strFilter;
		LoadFileDialogFilter(strFilter, IDS_INPUT_FILTER);
		CMultiFileDialog dlg(_T("ts"), m_szInputPath, OFN_HIDEREADONLY,
				strFilter);
		dlg.m_ofn.lpstrInitialDir = m_Settings.m_LastInputPath;
		if (dlg.DoModal() == IDOK) {
			CString strFile;
			bool find = dlg.GetFirstPathName(strFile);
			if (find) {
				dlg.GetDirectory(m_Settings.m_LastInputPath);
				if (dlg.GetNextPathName(NULL, 0) == 0) {
					// one file
					ATLTRACE(_T("%s\n"), strFile);
				//	AddFile(strFile);
					m_szInputPath = strFile;
					GetOutputFile(m_szInputPath, m_szOutputPath);
				} else {
					// multiple files
					m_szInputPath.Empty();
					m_szOutputPath.Empty();
					do {
						ATLTRACE(_T("%s\n"), strFile);
						AddFile(strFile);
					} while	(dlg.GetNextPathName(strFile));
				}
			}
			DoDataExchange(DDX_LOAD);
		}
		return 0;
	}

	LRESULT OnBrowseOutput(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		DoDataExchange(DDX_SAVE);
		CString strFilter;
		LoadFileDialogFilter(strFilter, IDS_OUTPUT_FILTER);
		CFileDialog dlg(FALSE, NULL, m_szOutputPath, OFN_OVERWRITEPROMPT,
				strFilter);
		if (dlg.DoModal() == IDOK) {
			ATLTRACE(_T("%s\n"), dlg.m_szFileName);
			m_szOutputPath = dlg.m_szFileName;
			DoDataExchange(DDX_LOAD);
		}
		return 0;
	}

	LRESULT OnBrowseFolder(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		DoDataExchange(DDX_SAVE);
		CString strMsg;
		strMsg.LoadString(IDS_BROWSE_FOLDER);
		CFolderDialog dlg(NULL, strMsg,
				BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE);

		dlg.SetInitialFolder(m_szDefaultPath);
		if (dlg.DoModal() == IDOK) {
			m_szDefaultPath = dlg.GetFolderPath();
			DoDataExchange(DDX_LOAD);
		}
		return 0;
	}

	void OnComboSelChange(UINT uNotifyCode, int nID, CWindow wndCtl)
	{
		int nIndex = m_combo_enc.GetCurSel();
		if (nIndex != CB_ERR) {
			m_Settings.m_SelectNum = nIndex;
		}
	}
	
	LRESULT OnButtonEncSet(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		CEncSettingDlg dlg(m_Settings);
		if (dlg.DoModal() == IDOK) {
			m_combo_enc.ResetContent();
			int count = m_Settings.m_SettingList.GetSize();
			for (int i = 0; i < count; i++) {
				m_combo_enc.AddString(m_Settings.m_SettingList[i].m_SettingName);
			}
			m_combo_enc.SetCurSel(m_Settings.m_SelectNum);
			
			//
			
		}
		return 0;
	}

	LRESULT OnBatchAdd(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		if (m_Dragging) {
			return 0;
		}
		DoDataExchange(DDX_SAVE);
		if (m_szOutputPath.IsEmpty()) {
			AddFile(m_szInputPath);
		} else {
			AddFile(m_szInputPath, m_szOutputPath);
		}
		m_szInputPath.Empty();
		m_szOutputPath.Empty();
		DoDataExchange(DDX_LOAD);
		return 0;
	}

	LRESULT OnBatchDel(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		if (m_Dragging) {
			return 0;
		}
		int sel = m_batch_list.GetCurSel();
		if (sel != LB_ERR) {
			m_batch_list.DeleteString(sel);
			m_batch_list.SetCurSel(min(sel, m_batch_list.GetCount()-1));
		}
	/*
		int selcount = m_batch_list.GetSelCount();
		int *items = new int[selcount];
		selcount = m_batch_list.GetSelItems(selcount, items);
		for (int i = selcount - 1; i >= 0; i--) {
			m_batch_list.DeleteString(items[i]);
		}
		delete [] items;
	*/
	//	m_szInputPath.Empty();
	//	m_szOutputPath.Empty();
		DoDataExchange(DDX_LOAD);
		return 0;
	}

	LRESULT OnBatchReplace(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		if (m_Dragging) {
			return 0;
		}
		DoDataExchange(DDX_SAVE);
		int sel = m_batch_list.GetCurSel();
		if (sel != LB_ERR) {
			m_batch_list.DeleteString(sel);
		}
		if (m_szOutputPath.IsEmpty()) {
			AddFile(m_szInputPath);
		} else {
			AddFile(m_szInputPath, m_szOutputPath);
		}
		m_szInputPath.Empty();
		m_szOutputPath.Empty();
		DoDataExchange(DDX_LOAD);
		return 0;
	}

	LRESULT OnBatchDelAll(UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtl*/)
	{
		if (m_Dragging) {
			return 0;
		}
		if (m_batch_list.GetCount() == 0) {
			return 0;
		}
		CString strTitle, strMsg;
		strTitle.LoadString(IDR_MAINFRAME);
		strMsg.LoadString(IDS_BATCH_DEL_ALL);
		if (MessageBox(strMsg, strTitle, MB_YESNO) == IDNO) {
			return 0;
		}
		m_batch_list.ResetContent();
	//	m_szInputPath.Empty();
	//	m_szOutputPath.Empty();
		DoDataExchange(DDX_LOAD);
		return 0;
	}

	void OnBatchListSelChange(UINT uNotifyCode, int nID, CWindow wndCtl)
	{
		int sel = m_batch_list.GetCurSel();
		if (sel == LB_ERR) {
			return;
		}
		CString str;
		m_batch_list.GetText(sel, str);
		LoadListItem(str);
	}

	BOOL OnBeginDrag(int nCtlID, HWND hWndDragList, POINT ptCursor)
	{
		// J[\ʒũACe̍ɑ}}[N`
		m_batch_list.DrawInsert(m_batch_list.LBItemFromPt(ptCursor));
		m_batch_list.AddString(_T(""));	// add dummy string
		m_Dragging = TRUE;
		return TRUE;
	}

	void OnCancelDrag(int nCtlID, HWND hWndDragList, POINT ptCursor)
	{
		// }}[N
		if (m_Dragging) {
			m_batch_list.DrawInsert(-1);
			m_batch_list.DeleteString(m_batch_list.GetCount() - 1);	// delete dummy string
			m_Dragging = FALSE;
		}
	}

	int OnDragging(int nCtlID, HWND hWndDragList, POINT ptCursor)
	{
		// J[\ʒũACe̍ɑ}}[N`
		m_batch_list.DrawInsert(m_batch_list.LBItemFromPt(ptCursor));
		return 0;
	}

	void OnDropped(int nCtlID, HWND hWndDragList, POINT ptCursor)
	{
		int nSrcIndex = m_batch_list.GetCurSel();				// ړ̃CfbNX
		int nDestIndex = m_batch_list.LBItemFromPt(ptCursor);	// ړ̃CfbNX

		OnCancelDrag(nCtlID, hWndDragList, ptCursor);	// Drag I

		if (nSrcIndex == -1 || nDestIndex == -1 || nDestIndex == nSrcIndex) {
			return;
		}

		CString strText;
		m_batch_list.GetText(nSrcIndex, strText);
		m_batch_list.DeleteString(nSrcIndex);

		if (nSrcIndex < nDestIndex) {
			--nDestIndex;
		}
		m_batch_list.InsertString(
				(nDestIndex == m_batch_list.GetCount()) ? -1 : nDestIndex,
				strText);
		m_batch_list.SetCurSel(nDestIndex);
	}

	// IProcessEventCallback
	void ProcessExit(DWORD ExitCode, bool Stopped)
	{
		ATLTRACE(_T("ProcessExit()\n"));
		PostMessage(WM_PROCESS_EXIT, (WPARAM) ExitCode, (LPARAM) Stopped);
	}
	LRESULT OnProcessExit(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		DWORD ExitCode = static_cast<DWORD>(wParam);
		bool Stopped = static_cast<bool>(lParam);
		ATLTRACE(_T("OnProcessExit() ExitCode:%d, Stopped:%d\n"), ExitCode, Stopped);
		
		if (Stopped) {
			// Stopped by the user.
			UnlockUI();
			return TRUE;
		}
		
		int sel = m_batch_list.GetCurSel();
		if (0 /*ExitCode == 0*/) {
			// Delete from the batch list when successfully converted.
			m_batch_list.DeleteString(sel);
			m_batch_list.SetCurSel(sel);
		} else {
			// Go to the next file.
			m_batch_list.SetCurSel(++sel);
		}
		if (m_batch_list.GetCurSel() != sel) {
			// Finished converting
			UnlockUI();
			FlashWindow(TRUE);
		} else {
			// next file
			StartConvert();
		}
		return TRUE;
	}
	
	// IProcessEventCallback
	void Progress(LPCTSTR msg, int pos, int total)
	{
		if (msg != NULL) {
			GetDlgItem(IDC_STATUS).SetWindowText(msg);
		}
		if ((pos >= 0) && (total > 0)) {
			m_progress.SetPos(pos * 1000 / total);
			if (static_cast<ITaskbarList3*>(m_ptrTaskbarList3)) {
				if (pos == 0) {
					m_ptrTaskbarList3->SetProgressState(m_hWnd, TBPF_NOPROGRESS);
				} else {
					m_ptrTaskbarList3->SetProgressValue(m_hWnd, pos, total);
				}
			}
		}
	}
	
	void LockUI() { LockUI(TRUE); }
	void UnlockUI() { LockUI(FALSE); }
	void LockUI(BOOL lock)
	{
		m_Converting = lock;
		CString strText;
		if (lock) {
			strText.LoadString(IDS_BUTTON_STOP);
		} else {
			strText.LoadString(IDS_BUTTON_START);
		}
		GetDlgItem(IDOK).SetWindowText(strText);
		
		BOOL enable = !lock;
		
		UINT uSkipIDs[] = {
			IDOK, IDC_STATUS, IDC_PROGRESS, IDC_STATIC,
		};
		for (HWND hWndChild = ::GetTopWindow(m_hWnd); hWndChild != NULL;
				hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT)) {
			bool skip = false;
			for (UINT i = 0; i < _countof(uSkipIDs); i++) {
				if (::GetDlgCtrlID(hWndChild) == uSkipIDs[i]) {
					skip = true;
				}
			}
			if (!skip) {
				::EnableWindow(hWndChild, enable);
			}
		}
	}

	void CloseDialog(int nVal)
	{
		DestroyWindow();
		::PostQuitMessage(nVal);
	}

	void AddFile(const CString& strInputFile)
	{
		CString strOutputFile;
		GetOutputFile(strInputFile, strOutputFile);
		AddFile(strInputFile, strOutputFile);
	}
	void AddFile(const CString& strInputFile, const CString& strOutputFile)
	{
		if (strInputFile.IsEmpty()) {
			return;
		}
		CString strSetting;
		m_combo_enc.GetLBText(m_combo_enc.GetCurSel(), strSetting);
		CString strAdd;
		strAdd.Format(_T("\"%s\",\"%s\",%s"), strInputFile, strOutputFile, strSetting);
		if (m_batch_list.FindStringExact(-1, strAdd) == -1) {
			m_batch_list.AddString(strAdd);
		}
	}
	void LoadListItem(const CString& strList)
	{
		int index1, index2;
		index1 = strList.Find(_T("\",\""), 1);
		index2 = strList.Find(_T("\","), index1 + 3);
		m_szInputPath = strList.Mid(1, index1 - 1);
		m_szOutputPath = strList.Mid(index1 + 3, index2 - (index1 + 3));
		CString strSetting = strList.Mid(index2 + 2);
		m_combo_enc.SetCurSel(m_combo_enc.FindStringExact(0, strSetting));
		DoDataExchange(DDX_LOAD);
	}

	void GetOutputFile(const CString& strInputFile, CString& strOutputFile)
	{
		LPTSTR buf;
		if (m_szDefaultPath.IsEmpty()) {
			strOutputFile = strInputFile;
			buf = strOutputFile.GetBuffer(MAX_PATH);
		} else {
			buf = strOutputFile.GetBuffer(MAX_PATH);
			::PathCombine(buf, m_szDefaultPath, ::PathFindFileName(strInputFile));
		}
		::PathRemoveExtension(buf);
		strOutputFile.ReleaseBuffer();
	}

	static void LoadFileDialogFilter(CString &strFilter, UINT nID)
	{
		strFilter.LoadString(nID);
		int len = strFilter.GetLength();
		LPTSTR buf = strFilter.GetBuffer(len);
		while (buf = _tcschr(buf, _T('|'))) {
			*buf++ = _T('\0');
		}
		strFilter.ReleaseBuffer(len);
	}
};
